import auditory.sampled.*;
import io.*;

import java.io.*;

/**
 * An app that renders sampled audio
 *
 * @author  Prof. David Bernstein, James Madison University
 * @see     "The Design and Implementation of Multimedia Software  2011"
 * @version 1.0
 */
public class BufferedSoundApp extends AbstractSampledAuditoryApp
{
    private BufferedSoundFactory      factory;       
    private ResourceFinder            finder;       


    /**
     * Initialize this App
     */
    public void init()
    {
       finder  = ResourceFinder.createInstance(this);
       factory = new BufferedSoundFactory(finder);

       super.init();       
    }
    


    /**
     * Create the BufferedSound
     * (required by AbstractSampledAuditoryApp)
     *
     * @return   The BufferedSound
     */
    protected BufferedSound createSound()
    {
       BufferedSound             a, b, sound;       
       BufferedSoundBinaryOp     binaryOp;       
       BufferedSoundUnaryOp      unaryOp;       
       String                    operatorType, value;       

       sound        = null;       
       operatorType = rootPaneContainer.getParameter("0");

       if      (operatorType == null)
       {
          sound = createSound("200");          
       }
       else if (operatorType.equalsIgnoreCase("NONE"))
       {
          value = rootPaneContainer.getParameter("1");
          sound = createSound(value);          
       }
       else if (operatorType.equalsIgnoreCase("UNARY"))
       {
          value    = rootPaneContainer.getParameter("1");
          a        = createSound(value);

          value    = rootPaneContainer.getParameter("2");
          unaryOp  = createUnaryOp(value);
          sound    = unaryOp.filter(a, null);
       }
       else if (operatorType.equalsIgnoreCase("BINARY"))
       {
          value    = rootPaneContainer.getParameter("1");
          a        = createSound(value);

          value    = rootPaneContainer.getParameter("2");
          binaryOp = createBinaryOp(value);

          value    = rootPaneContainer.getParameter("3");
          b        = createSound(value);

          sound    = binaryOp.filter(a, b, null);
       }
       


       return sound;
    }


    /**
     * Create a BufferedSound based on the value of a String (which may 
     * contain the frequency or the name of a file)
     */
    private BufferedSound createSound(String s)
    {
       BufferedSound    sound;       
       int              frequency;       

       sound     = null;
       frequency = 200;       
       
       try
       {
          frequency = Integer.parseInt(s);             
       }
       catch (NumberFormatException nfe)
       {
          try
          {
             sound   = factory.createBufferedSound(s);
          }
          catch (Exception e)
          {
             frequency = 200;
          }
       }
       
       if (sound == null)
       {
          sound = factory.createBufferedSound(frequency,
                                              500000,  // length
                                              4000.0f, // sampling rate
                                              1000.0f  // amplitude
                                              );
       }

       return sound;       
    }
    


    /**
     * Create a BufferedSoundBinaryOp from a String
     */
    private BufferedSoundBinaryOp createBinaryOp(String s)
    {
       BufferedSoundBinaryOp           op;
       
       op = null;
       
       if      (s == null)
       {
          op = new AddOp();          
       }
       else if (s.equalsIgnoreCase("ADD"))
       {
          op = new AddOp();          
       }
       else if (s.equalsIgnoreCase("MULTIPLY"))
       {
          op = new MultiplyOp();          
       }

       return op;       
    }
    
    


    /**
     * Create a BufferedSoundUnaryOp from a String
     */
    private BufferedSoundUnaryOp createUnaryOp(String s)
    {
       BufferedSoundUnaryOp           op;
       double[]                       weights;       
       FIRFilter                      firFilter;       
       int                            n;
       
       

       op = null;
       
       if      (s == null)
       {
          op = new NoiseOp(0.0);          
       }
       else if (s.equalsIgnoreCase("FIR"))
       {
          n       = 1000;
          weights = new double[n];

          // The weight on sample i is 1.0
          weights[0] = 1.0;
          
          // The weights on the "oldest" 1/4 of the samples
          for (int i=3*n/4; i<n; i++) weights[i] = 0.1;
          
          firFilter = new FIRFilter(weights);
          op        = new FIRFilterOp(firFilter);
       }
       else if (s.equalsIgnoreCase("INVERT"))
       {
          op = new InvertOp();          
       }
       else if (s.equalsIgnoreCase("NOISE"))
       {
          op = new NoiseOp(100.0);          
       }
       else if (s.equalsIgnoreCase("REVERSE"))
       {
          op = new ReverseOp();          
       }
       else if (s.equalsIgnoreCase("SPEEDCHANGE"))
       {
          op = new SpeedChangeOp(1.5);          
       }
       

       return op;       
    }
    
}

